]> git.xonotic.org Git - xonotic/darkplaces.git/blobdiff - cmd.h
Improve debug logging for render modules
[xonotic/darkplaces.git] / cmd.h
diff --git a/cmd.h b/cmd.h
index 336af38a063f20932fe00cf6088b296f60fcf062..c80fdf9078307f31f2f0afe5f9715af6fad25170 100644 (file)
--- a/cmd.h
+++ b/cmd.h
@@ -37,7 +37,9 @@ The game starts with a Cbuf_AddText ("exec quake.rc\n"); Cbuf_Execute ();
 #ifndef CMD_H
 #define CMD_H
 
-#include "thread.h"
+#include "qtypes.h"
+#include "qdefs.h"
+#include "com_list.h"
 
 struct cmd_state_s;
 
@@ -55,7 +57,7 @@ struct cmd_state_s;
 #define CF_USERINFO             (1<<9)  // command or cvar used to communicate userinfo to the server
 #define CF_PERSISTENT           (1<<10) // cvar must not be reset on gametype switch (such as scr_screenshot_name, which otherwise isn't set to the mod name properly)
 #define CF_PRIVATE              (1<<11) // cvar should not be $ expanded or sent to the server under any circumstances (rcon_password, etc)
-#define CF_MAXFLAGSVAL          4095    // used to determine if flags is valid
+#define CF_MAXFLAGSVAL          ((1<<12) - 1)    // used to determine if flags is valid
 // for internal use only!
 #define CF_DEFAULTSET (1<<30)
 #define CF_ALLOCATED (1<<31)
@@ -64,7 +66,7 @@ struct cmd_state_s;
 
 typedef void(*xcommand_t) (struct cmd_state_s *cmd);
 
-typedef enum
+typedef enum cmd_source_s
 {
        src_client,             ///< came in over a net connection as a clc_stringcmd
                                        ///< host_client will be valid during this state.
@@ -87,7 +89,7 @@ typedef struct cmd_function_s
        const char *name;
        const char *description;
        xcommand_t function;
-       qbool csqcfunc;
+       qbool qcfunc;
        qbool autofunc;
        qbool initstate; // indicates this command existed at init
 } cmd_function_t;
@@ -96,47 +98,60 @@ typedef struct cmd_function_s
 typedef struct cmd_userdefined_s
 {
        // csqc functions - this is a mess
-       cmd_function_t *csqc_functions;
+       cmd_function_t *qc_functions;
 
        // aliases
        cmd_alias_t *alias;
 }
 cmd_userdefined_t;
 
+typedef struct cmd_buf_s
+{
+       llist_t start;
+       llist_t deferred;
+       llist_t free;
+       qbool wait;
+       size_t maxsize;
+       size_t size;
+       char tokenizebuffer[CMD_TOKENIZELENGTH];
+       int tokenizebufferpos;
+       double deferred_oldtime;
+       void *lock;
+} cmd_buf_t;
+
 /// command interpreter state - the tokenizing and execution of commands, as well as pointers to which cvars and aliases they can access
 typedef struct cmd_state_s
 {
-       mempool_t *mempool;
+       struct mempool_s *mempool;
 
        int argc;
+       const char *cmdline;
        const char *argv[MAX_ARGS];
        const char *null_string;
        const char *args;
        cmd_source_t source;
 
-       struct cbuf_s *cbuf;
+       cmd_buf_t *cbuf;
 
        cmd_userdefined_t *userdefined; // possible csqc functions and aliases to execute
 
        cmd_function_t *engine_functions;
 
-       cvar_state_t *cvars; // which cvar system is this cmd state able to access? (&cvars_all or &cvars_null)
+       struct cvar_state_s *cvars; // which cvar system is this cmd state able to access? (&cvars_all or &cvars_null)
        int cvars_flagsmask; // which CVAR_* flags should be visible to this interpreter? (CF_CLIENT | CF_SERVER, or just CF_SERVER)
 
        int cmd_flags; // cmd flags that identify this interpreter
 
-       /*
-        * If a requested flag matches auto_flags, a command will be
-        * added to a given interpreter with auto_function. For example,
-        * a CF_SERVER_FROM_CLIENT command should be automatically added
-        * to the client interpreter as CL_ForwardToServer_f. It can be
-        * overridden at any time.
-        */
-       int auto_flags;
-       xcommand_t auto_function;
+       qbool (*Handle)(struct cmd_state_s *, struct cmd_function_s *, const char *, enum cmd_source_s);
+       qbool (*NotFound)(struct cmd_state_s *, struct cmd_function_s *, const char *, enum cmd_source_s);
 }
 cmd_state_t;
 
+qbool Cmd_Callback(cmd_state_t *cmd, cmd_function_t *func, const char *text, cmd_source_t src);
+qbool Cmd_CL_Callback(cmd_state_t *cmd, cmd_function_t *func, const char *text, cmd_source_t src);
+qbool Cmd_SV_Callback(cmd_state_t *cmd, cmd_function_t *func, const char *text, cmd_source_t src);
+qbool Cmd_SV_NotFound(cmd_state_t *cmd, cmd_function_t *func, const char *text, cmd_source_t src);
+
 typedef struct cmd_input_s
 {
        llist_t list;
@@ -148,37 +163,20 @@ typedef struct cmd_input_s
        qbool pending;
 } cmd_input_t;
 
-typedef struct cbuf_s
-{
-       llist_t start;
-       llist_t deferred;
-       llist_t free;
-       qbool wait;
-       size_t maxsize;
-       size_t size;
-       char tokenizebuffer[CMD_TOKENIZELENGTH];
-       int tokenizebufferpos;
-       double deferred_oldtime;
-       void *lock;
-} cbuf_t;
-
 extern cmd_userdefined_t cmd_userdefined_all; // aliases and csqc functions
 extern cmd_userdefined_t cmd_userdefined_null; // intentionally empty
 
// command interpreter for client commands injected by CSQC, MQC or client engine code
 // uses cmddefs_all
-extern cmd_state_t cmd_client;
-// command interpreter for server commands injected by MQC, SVQC, menu engine code or server engine code
-// uses cmddefs_all
-extern cmd_state_t cmd_server;
+extern cmd_state_t *cmd_local;
 // command interpreter for server commands received over network from clients
 // uses cmddefs_null
-extern cmd_state_t cmd_serverfromclient;
+extern cmd_state_t *cmd_serverfromclient;
 
 extern qbool host_stuffcmdsrun;
 
-void Cbuf_Lock(cbuf_t *cbuf);
-void Cbuf_Unlock(cbuf_t *cbuf);
+void Cbuf_Lock(cmd_buf_t *cbuf);
+void Cbuf_Unlock(cmd_buf_t *cbuf);
 
 /*! as new commands are generated from the console or keybindings,
  * the text is added to the end of the command buffer.
@@ -196,9 +194,9 @@ void Cbuf_InsertText (cmd_state_t *cmd, const char *text);
  * Normally called once per frame, but may be explicitly invoked.
  * \note Do not call inside a command function!
  */
-void Cbuf_Execute (cbuf_t *cbuf);
+void Cbuf_Execute (cmd_buf_t *cbuf);
 /*! Performs deferred commands and runs Cbuf_Execute, called by Host_Frame */
-void Cbuf_Frame (cbuf_t *cbuf);
+void Cbuf_Frame (cmd_buf_t *cbuf);
 
 //===========================================================================
 
@@ -226,8 +224,6 @@ void Cmd_AddCommand(int flags, const char *cmd_name, xcommand_t function, const
 // register commands and functions to call for them.
 // The cmd_name is referenced later, so it should not be in temp memory
 
-cmd_function_t *Cmd_GetCommand(cmd_state_t *cmd, const char *partial, size_t len, qbool casesensitive);
-
 /// used by the cvar code to check for cvar / command name overlap
 qbool Cmd_Exists (cmd_state_t *cmd, const char *cmd_name);
 
@@ -282,5 +278,7 @@ qbool Cmd_QuoteString(char *out, size_t outlen, const char *in, const char *quot
 
 void Cmd_ClearCSQCCommands (cmd_state_t *cmd);
 
+void Cmd_NoOperation_f(cmd_state_t *cmd);
+
 #endif
@@ -148,37 +163,20 @@ typedef struct cmd_input_s
 extern cmd_userdefined_t cmd_userdefined_null; // intentionally empty
 
-// command interpreter for client commands injected by CSQC, MQC or client engine code
+// command interpreter for local commands injected by SVQC, CSQC, MQC, server or client engine code
 // uses cmddefs_all